package net.jaekl.squelch.stmt;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Locale;

import net.jaekl.squelch.db.DbDriver;

public class Select extends Query {

	@Override
	public boolean handles(String line) {
		if (null == line) {
			return false;
		}
		
		String s = line.trim().toUpperCase(Locale.CANADA);
		return (s.startsWith("SELECT "));
	}

	@Override
	public int exec(DbDriver driver, Connection conn, PrintWriter pw, String line) throws IOException, SQLException 
	{
		int rowCount = 0;
		
		// If there's a ';' on the end of this line, remove it.
		String trimmed = line.trim();
		if (trimmed.endsWith(";")) {
			trimmed = trimmed.substring(0, trimmed.length() - 1);
		}
		
		try (PreparedStatement ps = conn.prepareStatement(trimmed)) 
		{
			try (ResultSet rs = ps.executeQuery()) 
			{
				rowCount = printFormatted(pw, rs);		
			}
		}
		return rowCount;
	}
	
	private int printFormatted(PrintWriter pw, ResultSet rs) throws IOException, SQLException
	{
		TabularResultSet trs = new TabularResultSet(rs);
		// TODO:  StringTable i18n
		int rowCount = trs.printTable(pw, "No rows returned.");
		pw.flush();
		
		return rowCount;
	}
}
